      SUBROUTINE PDPTS(DEG,PD1,PD2,WPD,NPD)
C
C     ******************************************************************
C
C                                                           From Padua2D
C
C                                       Marco Caliari and Marco Vianello
C                             Department of Pure and Applied Mathematics
C                                            University of Padua (Italy)
C                                        {mcaliari,marcov}@math.unipd.it
C
C                                                      Stefano De Marchi
C                                         Department of Computer Science
C                                           University of Verona (Italy)
C                                              stefano.demarchi@univr.it
C
C                                                        June 19th, 2007
C
C       This subroutine computes the first family of Padua points and 
C     weights corresponding to degree DEG.
C
C     ******************************************************************
C
C     PARAMS type                i=input, o=output, a=auxiliary           
C     
C     DEG    integer             i degree of approximation
C     PD1    double(*)           o array of the first coordinates of 
C                                  Padua points
C     PD2    double(*)           o array of the second coordinates of 
C                                  Padua points
C     WPD    double(*)           o array of the weights
C     NPD    integer             o number of Padua points = 
C                                  (DEG + 1) * (DEG + 2) / 2
C
C     ******************************************************************
C
      INTEGER DEG,NPD
      DOUBLE PRECISION PD1(*),PD2(*),WPD(*)
C
C     LOCALS
C
      INTEGER J,K,DEG1,ITEMP0,KOLD
      DOUBLE PRECISION PI,RTEMP0
      PARAMETER (PI = 3.1415926535897931D0)
C
C     Padua points of the first family
C
      IF (DEG .EQ. 0) THEN
C     
C     DEG .EQ. 0
C     
         PD1(1) = -1.D0
         PD2(1) = -1.D0
         WPD(1) = 2.0D0
         NPD = 1
         RETURN
      END IF   
      NPD = 0
      DEG1 = DEG + 1
      ITEMP0 = DEG * DEG1
      RTEMP0 = PI / ITEMP0
      K = 0
      NPD = NPD + 1
C     
C     The points are generated starting from bottom-left
C     and following the generating curve                           
C     
      PD1(NPD) = -1.d0
      PD2(NPD) = -1.d0
      WPD(NPD) = .5D0 / ITEMP0
      DO 10 K = 1,DEG - 1
         NPD = NPD + 1
         PD1(NPD) = -COS(DEG1 * K * RTEMP0)
         PD2(NPD) = -COS(DEG * K * RTEMP0)
         WPD(NPD) = 2.0D0 / ITEMP0
 10   CONTINUE
      K = DEG
      NPD = NPD + 1
      PD1(NPD) = -COS(DEG1 * K * RTEMP0)
      PD2(NPD) = -COS(DEG * K * RTEMP0)
      WPD(NPD) = 1.0D0 / ITEMP0
      KOLD = DEG1
      DO 20 J = 0,DEG - 2
         K = KOLD
         NPD = NPD + 1
         PD1(NPD) = -COS(DEG1 * K * RTEMP0)
         PD2(NPD) = -COS(DEG * K * RTEMP0)
         WPD(NPD) = 1.0D0 / ITEMP0
         DO 30 K = KOLD + 1,KOLD + DEG - J - 2
            NPD = NPD + 1
            PD1(NPD) = -COS(DEG1 * K * RTEMP0)
            PD2(NPD) = -COS(DEG * K * RTEMP0)
            WPD(NPD) = 2.0D0 / ITEMP0
 30      CONTINUE
         K = KOLD + DEG - J - 1
         NPD = NPD + 1
         PD1(NPD) = -COS(DEG1 * K * RTEMP0)
         PD2(NPD) = -COS(DEG * K * RTEMP0)
         WPD(NPD) = 1.0D0 / ITEMP0
         KOLD = KOLD + DEG1
 20   CONTINUE
      K = ITEMP0
      NPD = NPD + 1
      PD1(NPD) = -DBLE(2 * MOD(DEG,2) - 1)
      PD2(NPD) = -PD1(NPD)
      WPD(NPD) = 0.5D0 / ITEMP0
      RETURN
      END
